==> klayout-0.9.tar.gz: klayout-0.9/src/edtService.cc <==

/*

  KLayout Layout Viewer
  Copyright (C) 2006 Matthias Koefferlein

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/


#include "edtService.h"
#include "edtPropertiesPages.h"
#include "layLayoutView.h"
#include "tlProgress.h"

namespace edt
{

// -------------------------------------------------------------

//  search range for select in pixels
unsigned int search_range = 5;   // @@@ make variable?

//  max. number of tries in single-click selection before giving up
int point_sel_tests = 10000;

// -------------------------------------------------------------

class ShapeFinder
  : public Finder
{
public:
  struct StopException { };

  typedef std::vector<edt::Service::ShapeInstPath> founds_vector_type;
  typedef founds_vector_type::const_iterator iterator;

  ShapeFinder (bool point_mode, db::ShapeIterator::flags_type flags)
    : Finder (point_mode), m_flags (flags), m_progress (tl::translate ("Selecting ..."))
  {
    m_progress.set_unit (1000);
    m_progress.set_format ("");
  }

  bool find (lay::LayoutView *view, unsigned int cv_index, const db::Layout &layout, const db::Box &region, const db::Cell &cell, unsigned int l)
  {
    m_cv_index = cv_index;
    m_tries = point_sel_tests;

    //  actually find
    try {
      start (view, cv_index, layout, region, cell, view->min_hier_levels (), view->max_hier_levels (), int (l));
    } catch (StopException) {
      return false;
    }

    //  return true if anything was found
    return ! m_founds.empty ();
  }

  iterator begin () const
  {
    return m_founds.begin ();
  }

  iterator end () const
  {
    return m_founds.end ();
  }

private:
  virtual void 
  visit_cell (const db::Cell &cell, const db::Box &search_box, const db::MagTrans &t, int /*level*/)
  {
    if (! point_mode ()) {

      ++m_progress;

      const db::Shapes &shapes = cell.shapes (layer ());

      db::ShapeIterator shape = shapes.begin_touching (search_box, m_flags);
      while (! shape.at_end ()) {

        ++m_progress;

        //  in box mode, just test the boxes
        if (shape->bbox ().inside (search_box)) {

          //  in point mode just store that found that has the least "distance"
          m_founds.push_back (edt::Service::ShapeInstPath ());
          m_founds.back ().cv_index = m_cv_index;
          m_founds.back ().cell = &cell;
          m_founds.back ().path = path ();
          m_founds.back ().trans = t;
          m_founds.back ().layer = layer ();
          m_founds.back ().shape = *shape;

        }

        ++shape;

      } 

    } else {

      if (--m_tries < 0) {
        throw StopException ();
      }

      const db::Shapes &shapes = cell.shapes (layer ());

      db::ShapeIterator shape = shapes.begin_touching (search_box, m_flags);
      while (! shape.at_end ()) {

        bool match = false;
        double d = std::numeric_limits<double>::max ();

        if (--m_tries < 0) {
          throw StopException ();
        }

        db::Point point (search_box.center ());

        //  in point mode, test the edges and use a "closest" criterion
        if (shape->is_polygon ()) {

          for (db::Shape::polygon_edge_iterator e = shape->begin_edge (); ! e.at_end (); ++e) {
            test_edge (t * *e, d, match);
          }

          //  test if inside the polygon
          if (! match && inside_poly (shape->begin_edge (), point) >= 0) { 
            d = t (poly_dist (shape->begin_edge (), point)); 
            match = true;
          }

        } else if (shape->is_path ()) {

          //  test the "spine"
          db::Shape::point_iterator pt = shape->begin_point (); 
          if (pt != shape->end_point ()) {
            db::Point p (*pt);
            ++pt;
            for (; pt != shape->end_point (); ++pt) {
              test_edge (t * db::Edge (p, *pt), d, match);
              p = *pt;
            }
          }

          //  convert to polygon and test those edges
          db::Polygon poly;
          shape->polygon (poly);
          for (db::Polygon::polygon_edge_iterator e = poly.begin_edge (); ! e.at_end (); ++e) {
            test_edge (t * *e, d, match);
          }

          //  test if inside the polygon
          if (! match && inside_poly (poly.begin_edge (), point) >= 0) { 
            d = t (poly_dist (poly.begin_edge (), point));
            match = true;
          }

        } else if (shape->is_box ()) {

          const db::Box &box = shape->box ();

          //  convert to polygon and test those edges
          db::Polygon poly (box);
          for (db::Polygon::polygon_edge_iterator e = poly.begin_edge (); ! e.at_end (); ++e) {
            test_edge (t * *e, d, match);
          }

          if (! match && box.contains (search_box.center ())) {
            d = t (poly_dist (poly.begin_edge (), point)); 
            match = true;
          }

        } else if (shape->is_text ()) {

          db::Point tp (shape->text_trans () * db::Point ());
          if (search_box.contains (tp)) {
            d = tp.distance (search_box.center ());
            match = true;
          }

        }

        if (match && closer (d)) {

          //  in point mode just store that found that has the least "distance"
          if (m_founds.empty ()) {
            m_founds.push_back (edt::Service::ShapeInstPath ());
          }

          m_founds.back ().cv_index = m_cv_index;
          m_founds.back ().cell = &cell;
          m_founds.back ().path = path ();
          m_founds.back ().trans = t;
          m_founds.back ().layer = layer ();
          m_founds.back ().shape = *shape;

        }

        ++shape;

      }

    }

  }

  std::vector<edt::Service::ShapeInstPath> m_founds;
  db::ShapeIterator::flags_type m_flags;
  unsigned int m_cv_index;
  int m_tries;
  tl::Progress m_progress;
};


// -------------------------------------------------------------

Service::Service (db::Manager *manager, lay::LayoutView *view, db::ShapeIterator::flags_type flags)
  : lay::ViewService (view->canvas ()), 
    lay::Editable (view),
    lay::Plugin (view),
    db::Object (manager),
    mp_view (view),
    m_flags (flags)
{ 
  //  .. nothing yet ..
}

Service::~Service ()
{
  for (std::vector<lay::ShapeMarker *>::iterator r = m_markers.begin (); r != m_markers.end (); ++r) {
    delete *r;
  }
  m_markers.clear ();
}

void 
Service::clear_highlights ()
{
  for (std::vector<lay::ShapeMarker *>::iterator r = m_markers.begin (); r != m_markers.end (); ++r) {
    (*r)->visible (false);
  }
}

void 
Service::restore_highlights ()
{
  for (std::vector<lay::ShapeMarker *>::iterator r = m_markers.begin (); r != m_markers.end (); ++r) {
    (*r)->visible (true);
  }
}

void 
Service::highlight (unsigned int n)
{
  for (std::vector<lay::ShapeMarker *>::iterator r = m_markers.begin (); r != m_markers.end (); ++r) {
    (*r)->visible (n-- == 0);
  }
}

bool 
Service::has_selection ()
{
  return ! m_selection.empty ();
}

double 
Service::click_proximity (const db::DPoint &pos)
{
  ShapeFinder finder (true, m_flags);

  //  go through all visible layers of all cellviews
  for (lay::LayoutView::const_layer_iterator lprop = mp_view->begin_layers (); lprop != mp_view->end_layers (); ++lprop) {

    if (lprop->display.visible) {

      const lay::CellView &cv = mp_view->cellview (lprop->cellview_index);
      if (cv.is_valid ()) {

        double dbu = cv->layout ().dbu ();

        //  compute search box
        double l = double (search_range) / canvas ()->trans ().t2.factor ();
        db::Box search_box = db::Box::from_double (db::DMag (1.0 / dbu) * db::DBox (pos, pos).enlarged (db::DPoint (l, l)));

        finder.find (mp_view, lprop->cellview_index, cv->layout (), search_box, *cv.cell (), lprop->layer_index);

      }

    }

  }

  //  Return the finder's proximity value
  if (finder.begin () != finder.end ()) {
    return finder.proximity ();
  } else {
    return lay::Editable::click_proximity (pos); 
  }
}

bool 
Service::select (const db::DBox &box, lay::Editable::SelectionMode mode)
{
  bool needs_update = false;
  bool any_selected = false;

  //  clear before unless "add" is selected
  if (mode == lay::Editable::Replace) {
    if (! m_selection.empty ()) {
      m_selection.clear ();
      needs_update = true;
    }
  }

  if (box.empty ()) {

    //  unconditional selection
    if (mode == lay::Editable::Reset) {
      if (! m_selection.empty ()) {
        m_selection.clear ();
        needs_update = true;
      }
    } else {

      //  extract all shapes

      //  @@@ not implemented yet @@@

    }

  } else {

    ShapeFinder finder (box.is_point (), m_flags);

    //  go through all visible layers of all cellviews
    for (lay::LayoutView::const_layer_iterator lprop = mp_view->begin_layers (); lprop != mp_view->end_layers (); ++lprop) {

      //  box-selection
      if (lprop->display.visible) {

        const lay::CellView &cv = mp_view->cellview (lprop->cellview_index);
        if (cv.is_valid ()) {

          double dbu = cv->layout ().dbu ();

          //  compute search box
          double l = double (search_range) / canvas ()->trans ().t2.factor ();
          db::Box search_box = db::Box::from_double (db::DMag (1.0 / dbu) * box.enlarged (db::DPoint (l, l)));

          finder.find (mp_view, lprop->cellview_index, cv->layout (), search_box, *cv.cell (), lprop->layer_index);

        }

      }

    }

    //  collect the founds from the finder
    for (ShapeFinder::iterator f = finder.begin (); f != finder.end (); ++f) {
      select (*f, mode);
      needs_update = true;
      any_selected = true;
    }
      
  }

  //  if required, update the list of ruler objects to display the selection
  if (needs_update) {
    selection_to_view ();
  }

  return any_selected;
}

lay::PropertiesPage *
Service::properties_page (QWidget *parent)
{
  if (m_flags == db::ShapeIterator::Polygons) {
    return new edt::PolygonPropertiesPage (this, parent);
  } else if (m_flags == db::ShapeIterator::Boxes) {
    return new edt::BoxPropertiesPage (this, parent);
  } else if (m_flags == db::ShapeIterator::Paths) {
    return new edt::PathPropertiesPage (this, parent);
  } else if (m_flags == db::ShapeIterator::Texts) {
    return new edt::TextPropertiesPage (this, parent);
  } else {
    return 0;
  }
}

void 
Service::get_selection (std::vector <ShapeInstPath> &sel) const
{
  sel.clear ();
  sel.reserve (m_selection.size ());

  //  positions will hold a set of iterators that are to be erased
  for (std::set<ShapeInstPath>::const_iterator r = m_selection.begin (); r != m_selection.end (); ++r) {
    sel.push_back (*r);
  }
}

bool 
Service::configure (const std::string &name, const std::string &value)
{

  // @@@
  return false;

}

bool 
Service::select (const ShapeInstPath &obj, lay::Editable::SelectionMode mode)
{
  if (mode == lay::Editable::Replace || mode == lay::Editable::Add) {
    //  select
    if (m_selection.find (obj) == m_selection.end ()) {
      m_selection.insert (obj);
      return true;
    }
  } else if (mode == lay::Editable::Reset) {
    //  unselect
    if (m_selection.find (obj) != m_selection.end ()) {
      m_selection.erase (obj);
      return true;
    }
  } else {
    //  invert selection
    if (m_selection.find (obj) != m_selection.end ()) {
      m_selection.erase (obj);
    } else {
      m_selection.insert (obj);
    }
    return true;
  }
  return false;
}

void 
Service::clear_selection ()
{
  select (db::DBox (), lay::Editable::Reset);
}

void 
Service::selection_to_view ()
{
  //  the selection objects need to be recreated since we destroyed the old rulers
  for (std::vector<lay::ShapeMarker *>::iterator r = m_markers.begin (); r != m_markers.end (); ++r) {
    delete *r;
  }
  m_markers.clear ();
  m_markers.reserve (m_selection.size ());
  for (std::set<ShapeInstPath>::iterator r = m_selection.begin (); r != m_selection.end (); ++r) {
    m_markers.push_back (new lay::ShapeMarker (mp_view, r->cv_index));
    m_markers.back ()->set (r->shape, r->trans);
  }
}

// -------------------------------------------------------------

} // namespace edt

==> klayout-0.10.tar.gz: klayout-0.10/src/edtService.cc <==

/*

  KLayout Layout Viewer
  Copyright (C) 2006 Matthias Koefferlein

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/


#include "edtService.h"
#include "edtPropertiesPages.h"
#include "layLayoutView.h"
#include "tlProgress.h"

namespace edt
{

// -------------------------------------------------------------

//  search range for select in pixels
unsigned int search_range = 5;   // @@@ make variable?

//  max. number of tries in single-click selection before giving up
int point_sel_tests = 10000;

// -------------------------------------------------------------

class ShapeFinder
  : public Finder
{
public:
  struct StopException { };

  typedef std::vector<edt::Service::ShapeInstPath> founds_vector_type;
  typedef founds_vector_type::const_iterator iterator;

  ShapeFinder (bool point_mode, db::ShapeIterator::flags_type flags)
    : Finder (point_mode), m_flags (flags), mp_prop_sel (0), m_inv_prop_sel (false), m_progress (tl::translate ("Selecting ..."))
  {
    m_progress.set_unit (1000);
    m_progress.set_format ("");
  }

  bool find (lay::LayoutView *view, const lay::LayerProperties &lprops, const db::DBox &region_mu)
  {
    m_cv_index = lprops.cellview_index ();
    const lay::CellView &cv = view->cellview (m_cv_index);
    if (! cv.is_valid ()) {
      return false;
    }

    double dbu = cv->layout ().dbu ();
    db::Box region = db::Box::from_double (db::DMag (1.0 / dbu) * region_mu);

    m_tries = point_sel_tests;
    mp_prop_sel = &lprops.prop_sel ();
    m_inv_prop_sel = lprops.inverse_prop_sel ();

    //  actually find
    try {
      start (view, m_cv_index, lprops.trans (), cv->layout (), region, *cv.cell (), view->min_hier_levels (), view->max_hier_levels (), int (lprops.layer_index ()));
    } catch (StopException) {
      return false;
    }

    //  return true if anything was found
    return ! m_founds.empty ();
  }

  iterator begin () const
  {
    return m_founds.begin ();
  }

  iterator end () const
  {
    return m_founds.end ();
  }

private:
  virtual void 
  visit_cell (const db::Cell &cell, const db::Box &search_box, const db::MagTrans &t, int /*level*/)
  {
    if (! point_mode ()) {

      ++m_progress;

      const db::Shapes &shapes = cell.shapes (layer ());

      db::ShapeIterator shape = shapes.begin_touching (search_box, m_flags, mp_prop_sel, m_inv_prop_sel);
      while (! shape.at_end ()) {

        ++m_progress;

        //  in box mode, just test the boxes
        if (shape->bbox ().inside (search_box)) {

          //  in point mode just store that found that has the least "distance"
          m_founds.push_back (edt::Service::ShapeInstPath ());
          m_founds.back ().cv_index = m_cv_index;
          m_founds.back ().cell = &cell;
          m_founds.back ().path = path ();
          m_founds.back ().trans = t;
          m_founds.back ().layer = layer ();
          m_founds.back ().shape = *shape;

        }

        ++shape;

      } 

    } else {

      if (--m_tries < 0) {
        throw StopException ();
      }

      const db::Shapes &shapes = cell.shapes (layer ());

      db::ShapeIterator shape = shapes.begin_touching (search_box, m_flags, mp_prop_sel, m_inv_prop_sel);
      while (! shape.at_end ()) {

        bool match = false;
        double d = std::numeric_limits<double>::max ();

        if (--m_tries < 0) {
          throw StopException ();
        }

        db::Point point (search_box.center ());

        //  in point mode, test the edges and use a "closest" criterion
        if (shape->is_polygon ()) {

          for (db::Shape::polygon_edge_iterator e = shape->begin_edge (); ! e.at_end (); ++e) {
            test_edge (t * *e, d, match);
          }

          //  test if inside the polygon
          if (! match && inside_poly (shape->begin_edge (), point) >= 0) { 
            d = t.ctrans (poly_dist (shape->begin_edge (), point)); 
            match = true;
          }

        } else if (shape->is_path ()) {

          //  test the "spine"
          db::Shape::point_iterator pt = shape->begin_point (); 
          if (pt != shape->end_point ()) {
            db::Point p (*pt);
            ++pt;
            for (; pt != shape->end_point (); ++pt) {
              test_edge (t * db::Edge (p, *pt), d, match);
              p = *pt;
            }
          }

          //  convert to polygon and test those edges
          db::Polygon poly;
          shape->polygon (poly);
          for (db::Polygon::polygon_edge_iterator e = poly.begin_edge (); ! e.at_end (); ++e) {
            test_edge (t * *e, d, match);
          }

          //  test if inside the polygon
          if (! match && inside_poly (poly.begin_edge (), point) >= 0) { 
            d = t.ctrans (poly_dist (poly.begin_edge (), point));
            match = true;
          }

        } else if (shape->is_box ()) {

          const db::Box &box = shape->box ();

          //  convert to polygon and test those edges
          db::Polygon poly (box);
          for (db::Polygon::polygon_edge_iterator e = poly.begin_edge (); ! e.at_end (); ++e) {
            test_edge (t * *e, d, match);
          }

          if (! match && box.contains (search_box.center ())) {
            d = t.ctrans (poly_dist (poly.begin_edge (), point)); 
            match = true;
          }

        } else if (shape->is_text ()) {

          db::Point tp (shape->text_trans () * db::Point ());
          if (search_box.contains (tp)) {
            d = tp.distance (search_box.center ());
            match = true;
          }

        }

        if (match && closer (d)) {

          //  in point mode just store that found that has the least "distance"
          if (m_founds.empty ()) {
            m_founds.push_back (edt::Service::ShapeInstPath ());
          }

          m_founds.back ().cv_index = m_cv_index;
          m_founds.back ().cell = &cell;
          m_founds.back ().path = path ();
          m_founds.back ().trans = t;
          m_founds.back ().layer = layer ();
          m_founds.back ().shape = *shape;

        }

        ++shape;

      }

    }

  }

  std::vector<edt::Service::ShapeInstPath> m_founds;
  db::ShapeIterator::flags_type m_flags;
  unsigned int m_cv_index;
  const std::set<db::PropertiesRepository::properties_id_type> *mp_prop_sel;
  bool m_inv_prop_sel;
  int m_tries;
  tl::Progress m_progress;
};


// -------------------------------------------------------------

Service::Service (db::Manager *manager, lay::LayoutView *view, db::ShapeIterator::flags_type flags)
  : lay::ViewService (view->canvas ()), 
    lay::Editable (view),
    lay::Plugin (view),
    db::Object (manager),
    mp_view (view),
    m_flags (flags)
{ 
  //  .. nothing yet ..
}

Service::~Service ()
{
  for (std::vector<lay::ShapeMarker *>::iterator r = m_markers.begin (); r != m_markers.end (); ++r) {
    delete *r;
  }
  m_markers.clear ();
}

void 
Service::clear_highlights ()
{
  for (std::vector<lay::ShapeMarker *>::iterator r = m_markers.begin (); r != m_markers.end (); ++r) {
    (*r)->visible (false);
  }
}

void 
Service::restore_highlights ()
{
  for (std::vector<lay::ShapeMarker *>::iterator r = m_markers.begin (); r != m_markers.end (); ++r) {
    (*r)->visible (true);
  }
}

void 
Service::highlight (unsigned int n)
{
  for (std::vector<lay::ShapeMarker *>::iterator r = m_markers.begin (); r != m_markers.end (); ++r) {
    (*r)->visible (n-- == 0);
  }
}

bool 
Service::has_selection ()
{
  return ! m_selection.empty ();
}

double 
Service::click_proximity (const db::DPoint &pos)
{
  ShapeFinder finder (true, m_flags);

  //  go through all visible layers of all cellviews
  for (lay::LayerPropertiesConstIterator lprop = mp_view->begin_layers (); ! lprop.at_end () ; ++lprop) {

    if (lprop->visible (true /*real*/)) {

      //  compute search box
      double l = double (search_range) / (canvas ()->trans () * lprop->trans ()).t2.factor ();
      db::DBox search_box = db::DBox (pos, pos).enlarged (db::DPoint (l, l));

      finder.find (mp_view, *lprop, search_box);

    }

  }

  //  Return the finder's proximity value
  if (finder.begin () != finder.end ()) {
    return finder.proximity ();
  } else {
    return lay::Editable::click_proximity (pos); 
  }
}

bool 
Service::select (const db::DBox &box, lay::Editable::SelectionMode mode)
{
  bool needs_update = false;
  bool any_selected = false;

  //  clear before unless "add" is selected
  if (mode == lay::Editable::Replace) {
    if (! m_selection.empty ()) {
      m_selection.clear ();
      needs_update = true;
    }
  }

  if (box.empty ()) {

    //  unconditional selection
    if (mode == lay::Editable::Reset) {
      if (! m_selection.empty ()) {
        m_selection.clear ();
        needs_update = true;
      }
    } else {

      //  extract all shapes

      //  @@@ not implemented yet @@@

    }

  } else {

    ShapeFinder finder (box.is_point (), m_flags);

    //  go through all visible layers of all cellviews
    for (lay::LayerPropertiesConstIterator lprop = mp_view->begin_layers (); ! lprop.at_end (); ++lprop) {

      //  box-selection
      if (lprop->visible (true /*real*/)) {

        //  compute search box
        double l = double (search_range) / (canvas ()->trans () * lprop->trans ()).t2.factor ();
        db::DBox search_box = box.enlarged (db::DPoint (l, l));

        finder.find (mp_view, *lprop, search_box);

      }

    }

    //  collect the founds from the finder
    for (ShapeFinder::iterator f = finder.begin (); f != finder.end (); ++f) {
      select (*f, mode);
      needs_update = true;
      any_selected = true;
    }
      
  }

  //  if required, update the list of ruler objects to display the selection
  if (needs_update) {
    selection_to_view ();
  }

  return any_selected;
}

lay::PropertiesPage *
Service::properties_page (QWidget *parent)
{
  if (m_flags == db::ShapeIterator::Polygons) {
    return new edt::PolygonPropertiesPage (this, parent);
  } else if (m_flags == db::ShapeIterator::Boxes) {
    return new edt::BoxPropertiesPage (this, parent);
  } else if (m_flags == db::ShapeIterator::Paths) {
    return new edt::PathPropertiesPage (this, parent);
  } else if (m_flags == db::ShapeIterator::Texts) {
    return new edt::TextPropertiesPage (this, parent);
  } else {
    return 0;
  }
}

void 
Service::get_selection (std::vector <ShapeInstPath> &sel) const
{
  sel.clear ();
  sel.reserve (m_selection.size ());

  //  positions will hold a set of iterators that are to be erased
  for (std::set<ShapeInstPath>::const_iterator r = m_selection.begin (); r != m_selection.end (); ++r) {
    sel.push_back (*r);
  }
}

bool 
Service::configure (const std::string &name, const std::string &value)
{

  // @@@
  return false;

}

bool 
Service::select (const ShapeInstPath &obj, lay::Editable::SelectionMode mode)
{
  if (mode == lay::Editable::Replace || mode == lay::Editable::Add) {
    //  select
    if (m_selection.find (obj) == m_selection.end ()) {
      m_selection.insert (obj);
      return true;
    }
  } else if (mode == lay::Editable::Reset) {
    //  unselect
    if (m_selection.find (obj) != m_selection.end ()) {
      m_selection.erase (obj);
      return true;
    }
  } else {
    //  invert selection
    if (m_selection.find (obj) != m_selection.end ()) {
      m_selection.erase (obj);
    } else {
      m_selection.insert (obj);
    }
    return true;
  }
  return false;
}

void 
Service::clear_selection ()
{
  select (db::DBox (), lay::Editable::Reset);
}

void 
Service::selection_to_view ()
{
  //  the selection objects need to be recreated since we destroyed the old rulers
  for (std::vector<lay::ShapeMarker *>::iterator r = m_markers.begin (); r != m_markers.end (); ++r) {
    delete *r;
  }
  m_markers.clear ();
  m_markers.reserve (m_selection.size ());
  for (std::set<ShapeInstPath>::iterator r = m_selection.begin (); r != m_selection.end (); ++r) {
    m_markers.push_back (new lay::ShapeMarker (mp_view, r->cv_index));
    m_markers.back ()->set (r->shape, r->trans);
  }
}

// -------------------------------------------------------------

} // namespace edt
